@@ -5,6 +5,7 @@ package e2e
 import (
 	"context"
 	"fmt"
+	"reflect"
 	"testing"
 	"time"
 
@@ -13,6 +14,7 @@ import (
 
 	operatorclient "github.com/openshift/cluster-dns-operator/pkg/operator/client"
 	operatorcontroller "github.com/openshift/cluster-dns-operator/pkg/operator/controller"
+	statuscontroller "github.com/openshift/cluster-dns-operator/pkg/operator/controller/status"
 
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/config"
@@ -44,6 +46,15 @@ const (
 `
 )
 
+var (
+	dnsName = types.NamespacedName{Name: operatorcontroller.DefaultDNSName}
+	opName  = types.NamespacedName{Name: operatorcontroller.DefaultOperatorName}
+
+	defaultAvailableDNSConditions = []operatorv1.OperatorCondition{
+		{Type: operatorv1.OperatorStatusTypeAvailable, Status: operatorv1.ConditionTrue},
+	}
+)
+
 func getClient() (client.Client, error) {
 	// Get a kube client.
 	kubeConfig, err := config.GetConfig()
@@ -65,7 +76,8 @@ func TestOperatorAvailable(t *testing.T) {
 
 	err = wait.PollImmediate(1*time.Second, 5*time.Minute, func() (bool, error) {
 		co := &configv1.ClusterOperator{}
-		if err := cl.Get(context.TODO(), types.NamespacedName{Name: operatorcontroller.DNSOperatorName}, co); err != nil {
+		if err := cl.Get(context.TODO(), opName, co); err != nil {
+			t.Logf("failed to get DNS cluster operator %s: %v", opName.Name, err)
 			return false, nil
 		}
 
@@ -82,6 +94,46 @@ func TestOperatorAvailable(t *testing.T) {
 	}
 }
 
+func TestClusterOperatorStatusRelatedObjects(t *testing.T) {
+	cl, err := getClient()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	expected := []configv1.ObjectReference{
+		{
+			Resource: "namespaces",
+			Name:     "openshift-dns-operator",
+		},
+		{
+			Group:    operatorv1.GroupName,
+			Resource: "dnses",
+			Name:     "default",
+		},
+		{
+			Resource: "namespaces",
+			Name:     "openshift-dns",
+		},
+	}
+
+	err = wait.PollImmediate(1*time.Second, 5*time.Minute, func() (bool, error) {
+		co := &configv1.ClusterOperator{}
+		if err := cl.Get(context.TODO(), opName, co); err != nil {
+			t.Logf("failed to get DNS cluster operator %s: %v", opName.Name, err)
+			return false, nil
+		}
+
+		if reflect.DeepEqual(expected, co.Status.RelatedObjects) {
+			return true, nil
+		}
+
+		return false, nil
+	})
+	if err != nil {
+		t.Errorf("did not get expected status related objects: %v", err)
+	}
+}
+
 func TestDefaultDNSExists(t *testing.T) {
 	cl, err := getClient()
 	if err != nil {
@@ -90,7 +142,8 @@ func TestDefaultDNSExists(t *testing.T) {
 
 	err = wait.PollImmediate(1*time.Second, 5*time.Minute, func() (bool, error) {
 		dns := &operatorv1.DNS{}
-		if err := cl.Get(context.TODO(), types.NamespacedName{Name: "default"}, dns); err != nil {
+		if err := cl.Get(context.TODO(), dnsName, dns); err != nil {
+			t.Logf("failed to get DNS operator %s: %v", dnsName.Name, err)
 			return false, nil
 		}
 		return true, nil
@@ -100,68 +153,26 @@ func TestDefaultDNSExists(t *testing.T) {
 	}
 }
 
-func TestVersionReporting(t *testing.T) {
+func TestOperatorSteadyConditions(t *testing.T) {
 	cl, err := getClient()
 	if err != nil {
 		t.Fatal(err)
 	}
+	expected := []configv1.ClusterOperatorStatusCondition{
+		{Type: configv1.OperatorAvailable, Status: configv1.ConditionTrue},
+	}
+	if err := waitForClusterOperatorConditions(t, cl, 10*time.Second, expected...); err != nil {
+		t.Errorf("did not get expected available condition: %v", err)
+	}
+}
 
-	deployment := &appsv1.Deployment{}
-	namespacedName := types.NamespacedName{Namespace: "openshift-dns-operator", Name: "dns-operator"}
-	err = wait.PollImmediate(1*time.Second, 5*time.Minute, func() (bool, error) {
-		if err := cl.Get(context.TODO(), namespacedName, deployment); err != nil {
-			return false, nil
-		}
-		return true, nil
-	})
+func TestDefaultDNSSteadyConditions(t *testing.T) {
+	cl, err := getClient()
 	if err != nil {
-		t.Errorf("failed to get deployment: %v", err)
+		t.Fatal(err)
 	}
-
-	var curVersion string
-	for _, env := range deployment.Spec.Template.Spec.Containers[0].Env {
-		if env.Name == "RELEASE_VERSION" {
-			curVersion = env.Value
-			break
-		}
-	}
-	if len(curVersion) == 0 {
-		t.Errorf("env RELEASE_VERSION not found in the operator deployment")
-	}
-
-	newVersion := "0.0.1-test"
-	setVersion(deployment, newVersion)
-	if err := cl.Update(context.TODO(), deployment); err != nil {
-		t.Fatalf("failed to update dns operator to new version: %v", err)
-	}
-	defer func() {
-		if err := cl.Get(context.TODO(), namespacedName, deployment); err != nil {
-			t.Fatalf("failed to get latest deployment: %v", err)
-		}
-		setVersion(deployment, curVersion)
-		if err := cl.Update(context.TODO(), deployment); err != nil {
-			t.Fatalf("failed to restore dns operator to old release version: %v", err)
-		}
-	}()
-
-	err = wait.PollImmediate(1*time.Second, 5*time.Minute, func() (bool, error) {
-		co := &configv1.ClusterOperator{}
-		if err := cl.Get(context.TODO(), types.NamespacedName{Name: operatorcontroller.DNSOperatorName}, co); err != nil {
-			return false, nil
-		}
-
-		for _, v := range co.Status.Versions {
-			if v.Name == "operator" {
-				if v.Version == newVersion {
-					return true, nil
-				}
-				break
-			}
-		}
-		return false, nil
-	})
-	if err != nil {
-		t.Errorf("failed to observe updated version reported in dns clusteroperator status: %v", err)
+	if err := waitForDNSConditions(t, cl, 10*time.Second, dnsName, defaultAvailableDNSConditions...); err != nil {
+		t.Errorf("did not get expected conditions: %v", err)
 	}
 }
 
@@ -175,6 +186,7 @@ func TestCoreDNSImageUpgrade(t *testing.T) {
 	namespacedName := types.NamespacedName{Namespace: "openshift-dns-operator", Name: "dns-operator"}
 	err = wait.PollImmediate(1*time.Second, 5*time.Minute, func() (bool, error) {
 		if err := cl.Get(context.TODO(), namespacedName, deployment); err != nil {
+			t.Logf("failed to get deployment %s/%s: %v", namespacedName.Namespace, namespacedName.Name, err)
 			return false, nil
 		}
 		return true, nil
@@ -212,6 +224,7 @@ func TestCoreDNSImageUpgrade(t *testing.T) {
 	err = wait.PollImmediate(1*time.Second, 3*time.Minute, func() (bool, error) {
 		podList := &corev1.PodList{}
 		if err := cl.List(context.TODO(), podList, client.InNamespace("openshift-dns")); err != nil {
+			t.Logf("failed to get pod list in openshift-dns namespace: %v", err)
 			return false, nil
 		}
 
@@ -269,17 +282,17 @@ func TestDNSForwarding(t *testing.T) {
 
 	// Get the CoreDNS image used by the test upstream resolver.
 	co := &configv1.ClusterOperator{}
-	if err := cl.Get(context.TODO(), types.NamespacedName{Name: operatorcontroller.DNSOperatorName}, co); err != nil {
-		t.Fatalf("failed to get clusteroperator %s: %v", operatorcontroller.DNSOperatorName, err)
+	if err := cl.Get(context.TODO(), opName, co); err != nil {
+		t.Fatalf("failed to get clusteroperator %s: %v", opName, err)
 	}
 	var (
 		coreImage      string
 		coreImageFound bool
 	)
 	for _, ver := range co.Status.Versions {
-		if ver.Name == operatorcontroller.CoreDNSVersionName {
+		if ver.Name == statuscontroller.CoreDNSVersionName {
 			if len(ver.Version) == 0 {
-				t.Fatalf("clusteroperator %s has empty coredns version", operatorcontroller.DNSOperatorName)
+				t.Fatalf("clusteroperator %s has empty coredns version", opName)
 			}
 			coreImageFound = true
 			coreImage = ver.Version
@@ -287,7 +300,7 @@ func TestDNSForwarding(t *testing.T) {
 		}
 	}
 	if !coreImageFound {
-		t.Fatalf("version %s not found for clusteroperator %s", operatorcontroller.CoreDNSVersionName, operatorcontroller.DNSOperatorName)
+		t.Fatalf("version %s not found for clusteroperator %s", statuscontroller.CoreDNSVersionName, opName)
 	}
 
 	// Create the upstream resolver Pod.
@@ -302,8 +315,10 @@ func TestDNSForwarding(t *testing.T) {
 	}()
 
 	// Wait for the upstream resolver Pod to be ready.
+	name := types.NamespacedName{Namespace: upstreamResolver.Namespace, Name: upstreamResolver.Name}
 	err = wait.PollImmediate(1*time.Second, 2*time.Minute, func() (bool, error) {
-		if err := cl.Get(context.TODO(), types.NamespacedName{Namespace: upstreamResolver.Namespace, Name: upstreamResolver.Name}, upstreamResolver); err != nil {
+		if err := cl.Get(context.TODO(), name, upstreamResolver); err != nil {
+			t.Logf("failed to get pod %s/%s: %v", name.Namespace, name.Name, err)
 			return false, nil
 		}
 		for _, cond := range upstreamResolver.Status.Conditions {
@@ -391,7 +406,7 @@ func TestDNSForwarding(t *testing.T) {
 		cliImageFound bool
 	)
 	for _, ver := range co.Status.Versions {
-		if ver.Name == operatorcontroller.OpenshiftCLIVersionName {
+		if ver.Name == statuscontroller.OpenshiftCLIVersionName {
 			if len(ver.Version) == 0 {
 				break
 			}
@@ -401,7 +416,7 @@ func TestDNSForwarding(t *testing.T) {
 		}
 	}
 	if !cliImageFound {
-		t.Fatalf("failed to find the %s version for clusteroperator %s", operatorcontroller.OpenshiftCLIVersionName, co.Name)
+		t.Fatalf("failed to find the %s version for clusteroperator %s", statuscontroller.OpenshiftCLIVersionName, co.Name)
 	}
 
 	// Create the client Pod.
@@ -415,8 +430,10 @@ func TestDNSForwarding(t *testing.T) {
 		}
 	}()
 	// Wait for the client Pod to be ready.
+	name = types.NamespacedName{Namespace: testClient.Namespace, Name: testClient.Name}
 	err = wait.PollImmediate(1*time.Second, 60*time.Second, func() (bool, error) {
-		if err := cl.Get(context.TODO(), types.NamespacedName{Namespace: testClient.Namespace, Name: testClient.Name}, testClient); err != nil {
+		if err := cl.Get(context.TODO(), name, testClient); err != nil {
+			t.Logf("failed to get pod %s/%s: %v", name.Namespace, name.Name, err)
 			return false, nil
 		}
 		for _, cond := range testClient.Status.Conditions {
